#ifndef _PA_Sprite
#define _PA_Sprite


/*! \file PA_Sprite.h
    \brief Everything concerning the sprite system

    This file contains all macros, variables, and prototypes regarding the sprite system (OAM, Sprite mouvement, Gfx loading...)
*/

//#include "../PA.h"


#define EWRAM_DATA	__attribute__((section(".ewram")))
#define EWRAM_BSS	__attribute__((section(".sbss")))
#define PA_Cos(angle) PA_SIN[((angle) + 128)&511]
#define PA_Sin(angle) PA_SIN[((angle))&511]
extern const s16 PA_SIN[512]; 

#define MAX_DRAW 16  // Maximum 16 sprites dessinables...


//////////////////////////////////////////////////////////////////////
// Systme de sprites
//////////////////////////////////////////////////////////////////////


#define OBJ_SIZE_8X8      0,0
#define OBJ_SIZE_16X16    0,1
#define OBJ_SIZE_32X32    0,2
#define OBJ_SIZE_64X64    0,3
#define OBJ_SIZE_16X8     1,0
#define OBJ_SIZE_32X8     1,1
#define OBJ_SIZE_32X16    1,2
#define OBJ_SIZE_64X32    1,3
#define OBJ_SIZE_8X16     2,0
#define OBJ_SIZE_8X32     2,1
#define OBJ_SIZE_16X32    2,2
#define OBJ_SIZE_32X64    2,3

typedef struct {
	u8 lx, ly;
} PA_sizes;

extern const PA_sizes PA_size[3][4];


#define BITS_16 65535
#define ALL_BUT(ATR) (BITS_16 - ATR)

#define OBJ_X 511 //511, debugging...
#define OBJ_Y 255
#define OBJ_ROT 256
#define ALL_BUT_ROTSET 49663  // Numro du rotset
#define ALL_BUT_PAL 4095

#define DBLSIZE 512
#define N_COLORS (1 << 13)
#define OBJ_MODE 3072
#define OBJ_MOSAIC 4096
#define OBJ_HFLIP 4096
#define OBJ_VFLIP 8192

#define OBJ_GFX 1023
#define OBJ_PRIO 3072

#define COLORS_256 1
#define COLORS_16 0



#define MEM_DECAL 5
#define NUMBER_DECAL 7



// Mmoire gfx
typedef struct {
   u16 mem_block;  // Emplacement en mmoire, de 0  1023
   u16 free; // Doones libres en ce point
} mem_usage;

extern u16 n_free_mem[2]; // nombre d'emplacements libres
extern u8 used_mem[2][1024]; // Note la quantit de mmoire utilise en chaque point de la mmoire pour pouvoir effacer les gfx...
extern u8 obj_per_gfx[2][1024]; // Nombre de sprites utilisant un gfx donn...
extern mem_usage free_mem[2][1024];

extern u16 FirstGfx[2];

extern const u16 PA_obj_sizes[4][3];

typedef struct {
   u16 atr0, atr1, atr2, atr3;
}obj_inf;
extern obj_inf PA_obj[2][128];  // Les 128 premiers pour l'cran du haut, et encore 128 pour l'cran du bas...


// Infos pour dessiner sur les sprites

typedef struct {
	bool screen; // Ecran
	u8 sprite;		// Numro du sprite
	bool wasdrawing; // Si on dessinait dessus ou pas avant
	s8 x, y;  // Position x et y d'avant
	u8 hsize, vsize; // Tailles
	u8 n_colors; // Nombre de couleurs
	u8 drawsize;
	u32 *gfxpointer; // Pour dessiner  l'cran directement
} PA_DrawSprites;


extern PA_DrawSprites PA_DrawSprite[MAX_DRAW];

// Pour le dessin sur sprite
extern unsigned char *PA_SpriteBuffer[MAX_DRAW]; // Pointeurs vers les buffers  que l'on crera  la vole

extern u16 *PA_SpriteAnimP[2][128];



#define OAM0 0x07000000
#define OAM1 0x07000400


/** @defgroup Sprite Sprite system
 *  Load Sprite, move them around, rotate them...
 *  @{
 */

/*! \def PA_UpdateOAM()
    \brief
         \~english Update the sprite infos for both screens. Do this in the VBL
         \~french Mettre  jour les infos des sprites pour les 2 crans. A faire dans le VBL
*/

//#define PA_UpdateOAM() DMA_Copy((void*)PA_obj, (void*)OAM0, 512, DMA_32NOW)
#define PA_UpdateOAM()s16 i;\
    	for (i = 0; i < 256; i++) {\
		OAM[(i << 2)] = PA_obj[0][i].atr0;\
		OAM[1+(i << 2)] = PA_obj[0][i].atr1;\
		OAM[2+(i << 2)] = PA_obj[0][i].atr2;\
		OAM[3+(i << 2)] = PA_obj[0][i].atr3;}


/*! \def PA_UpdateOAM0()
    \brief
         \~english Update the sprite infos for screen 0 only. Do this in the VBL
         \~french Mettre  jour les infos des sprites pour l'cran 0 uniquement. A faire dans le VBL
*/
#define PA_UpdateOAM0() DMA_Copy((void*)PA_obj, (void*)OAM0, 256, DMA_32NOW)

/*! \def PA_UpdateOAM1()
    \brief
         \~english Update the sprite infos for screen 1 only. Do this in the VBL
         \~french Mettre  jour les infos des sprites pour l'cran 1 uniquement. A faire dans le VBL
*/
#define PA_UpdateOAM1() DMA_Copy((void*)PA_obj + 256, (void*)OAM1, 256, DMA_32NOW)


/*! \fn u16 PA_CreateGfx(bool screen, void* obj_data, u8 obj_shape, u8 obj_size, u8 color_mode)
    \brief
         \~english Load in mmory a gfx to use later on for a sprite. Returns the gfx's number in memory
         \~french Charger en mmoire un gfx  utiliser plus tard pour un sprite. Renvoie le numro en mmoire.
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj_data
         \~english Gfx to load
         \~french Gfx  charger
    \param obj_shape
         \~english Object shape, from 0 to 2. Use the OBJ_SIZE_32X32 (...) macros for object shape and obj_size...
         \~french Forme du sprite  charger, de 0  2. Utiliser la macro OBJ_SIZE_32X32 (...) pour charger la forme et la taille...
    \param obj_size
         \~english Object size. Use the OBJ_SIZE_32X32 (...) macros for object shape and obj_size...
         \~french Taille du sprite. Utiliser la macro OBJ_SIZE_32X32 (...) pour charger la forme et la taille...
    \param color_mode
         \~english 256 or 16 color mode (1 or 0).
         \~french Mode 256 ou 16 couleurs (1 ou 0).
*/

u16 PA_CreateGfx(bool screen, void* obj_data, u8 obj_shape, u8 obj_size, u8 color_mode);

/*! \fn void PA_ResetSpriteSys(void)
    \brief
         \~english Reset the sprite system, memory, etc...
         \~french Remise  0 du systme de sprite, de la mmoire...
*/
void PA_ResetSpriteSys(void);



/*! \fn extern inline void PA_CreateSprite(bool screen, u8 obj_number, void* obj_data, u8 obj_shape, u8 obj_size, u8 color_mode, u8 palette, s16 x, s16 y)
    \brief
         \~english Create a sprite with it's gfx. This is the simple version of the function
         \~french Creer un sprite avec ses gfx... Ceci est la version simple de la fonction
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj_number
         \~english Object number you want to use (0-127 for each screen seperately). 
         \~french Numro du sprite que vous voulez utiliser (de 0  127 pour chaque cran sparemment).
    \param obj_data
         \~english Gfx to load
         \~french Gfx  charger
    \param obj_shape
         \~english Object shape, from 0 to 2. Use the OBJ_SIZE_32X32 (...) macros for object shape and obj_size...
         \~french Forme du sprite  charger, de 0  2. Utiliser la macro OBJ_SIZE_32X32 (...) pour charger la forme et la taille...
    \param obj_size
         \~english Object size. Use the OBJ_SIZE_32X32 (...) macros for object shape and obj_size...
         \~french Taille du sprite. Utiliser la macro OBJ_SIZE_32X32 (...) pour charger la forme et la taille...
    \param color_mode
         \~english 256 or 16 color mode (1 or 0).
         \~french Mode 256 ou 16 couleurs (1 ou 0).
     \param palette
         \~english Palette to use (0-15). Leave to 0 if in 256 color mode...
         \~french Palette  utiliser (0-15). Laisser  0 si en mode 256 couleurs.
    \param x
         \~english X position of the sprite
         \~french Position X du sprite
    \param y
         \~english Y position of the sprite
         \~french Position Y du sprite
*/
extern inline void PA_CreateSprite(bool screen, u8 obj_number, void* obj_data, u8 obj_shape, u8 obj_size, u8 color_mode, u8 palette, s16 x, s16 y) {
   PA_SpriteAnimP[screen][obj_number] = (u16*)obj_data; // Store for later use in animations !
   PA_obj[screen][obj_number].atr2 = PA_CreateGfx(screen, obj_data, obj_shape, obj_size, color_mode) + (palette << 12);
   PA_obj[screen][obj_number].atr0 = (y&OBJ_Y) + (color_mode << 13) + (obj_shape << 14);
   PA_obj[screen][obj_number].atr1 = (x & OBJ_X) + (obj_size << 14);
};

/*! \fn extern inline void PA_CreateSpriteEx(bool screen, u8 obj_number, void* obj_data, u8 obj_shape, u8 obj_size, u8 color_mode, u8 palette, u8 obj_mode, bool mosaic, bool hflip, bool vflip, u8 prio, bool dblsize, s16 x, s16 y)
    \brief
         \~english Create a sprite with it's gfx. This is the complex version of the function
         \~french Creer un sprite avec ses gfx... Ceci est la version complexe de la fonction
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj_number
         \~english Object number you want to use (0-127 for each screen seperately). 
         \~french Numro du sprite que vous voulez utiliser (de 0  127 pour chaque cran sparemment).
    \param obj_data
         \~english Gfx to load
         \~french Gfx  charger
    \param obj_shape
         \~english Object shape, from 0 to 2. Use the OBJ_SIZE_32X32 (...) macros for object shape and obj_size...
         \~french Forme du sprite  charger, de 0  2. Utiliser la macro OBJ_SIZE_32X32 (...) pour charger la forme et la taille...
    \param obj_size
         \~english Object size. Use the OBJ_SIZE_32X32 (...) macros for object shape and obj_size...
         \~french Taille du sprite. Utiliser la macro OBJ_SIZE_32X32 (...) pour charger la forme et la taille...
    \param color_mode
         \~english 256 or 16 color mode (1 or 0).
         \~french Mode 256 ou 16 couleurs (1 ou 0).
     \param palette
         \~english Palette to use (0-15). Leave to 0 if in 256 color mode...
         \~french Palette  utiliser (0-15). Laisser  0 si en mode 256 couleurs.
    \param obj_mode
         \~english Object mode (normal, transparent, window). Not functionnal yet, please leave to 0 for now
         \~french Mode du sprite (normal, transparent, fenetre). Pas encore oprationnel, laisser  0...
     \param mosaic
         \~english Activate Mosaic for the sprite or not. Not yet functionnal either :p
         \~french Activer le mode mosaique pour ce sprite. Pas encore au point...
    \param hflip
         \~english Horizontal flip on or off...
         \~french Flip horizontal activ ou non.
     \param vflip
         \~english Vertical flip...
         \~french Flip vertical...
    \param prio
         \~english Sprite priority regarding backgrounds : in front of which background to show it (0-3)
         \~french Priorit du sprite vis--vis des fonds : devant quel fond l'afficher... (0-3)
     \param dblsize
         \~english Double the possible sprite size. Activate only if you are going to rotate and zoom in the sprite
         \~french Doubler la taille possible du sprite. A activer uniquement si on compte grossir et faire tourner le sprite
    \param x
         \~english X position of the sprite
         \~french Position X du sprite
    \param y
         \~english Y position of the sprite
         \~french Position Y du sprite
*/
extern inline void PA_CreateSpriteEx(bool screen, u8 obj_number, void* obj_data, u8 obj_shape, u8 obj_size, u8 color_mode, u8 palette, u8 obj_mode, bool mosaic, bool hflip, bool vflip, u8 prio, bool dblsize, s16 x, s16 y) {
   PA_SpriteAnimP[screen][obj_number] = (u16*)obj_data;
   PA_obj[screen][obj_number].atr2 = PA_CreateGfx(screen, obj_data, obj_shape, obj_size, color_mode) + (prio << 10) + (palette << 12);
   PA_obj[screen][obj_number].atr0 = (y&OBJ_Y) + (dblsize << 9) + (obj_mode << 10) + (mosaic << 12) + (color_mode << 13) + (obj_shape << 14);
   PA_obj[screen][obj_number].atr1 = (x & OBJ_X) + (hflip << 12) + (vflip << 13) + (obj_size << 14);
};


/*! \fn extern inline void PA_CreateSpriteFromGfx(bool screen, u8 obj_number, u16 obj_gfx, u8 obj_shape, u8 obj_size, u8 color_mode, u8 palette, s16 x, s16 y)
    \brief
         \~english Create a sprite with it's gfx. This is the simple version of the function
         \~french Creer un sprite avec ses gfx... Ceci est la version simple de la fonction
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj_number
         \~english Object number you want to use (0-127 for each screen seperately). 
         \~french Numro du sprite que vous voulez utiliser (de 0  127 pour chaque cran sparemment).
    \param obj_gfx
         \~english Memory gfx to use. Get it by using PA_GetSpriteGfx or PA_CreateGfx
         \~french Gfx en mmoire  utiliser. On peut en avoir avec PA_GetSpriteGfx ou PA_CreateGfx
    \param obj_shape
         \~english Object shape, from 0 to 2. Use the OBJ_SIZE_32X32 (...) macros for object shape and obj_size...
         \~french Forme du sprite  charger, de 0  2. Utiliser la macro OBJ_SIZE_32X32 (...) pour charger la forme et la taille...
    \param obj_size
         \~english Object size. Use the OBJ_SIZE_32X32 (...) macros for object shape and obj_size...
         \~french Taille du sprite. Utiliser la macro OBJ_SIZE_32X32 (...) pour charger la forme et la taille...
    \param color_mode
         \~english 256 or 16 color mode (1 or 0).
         \~french Mode 256 ou 16 couleurs (1 ou 0).
     \param palette
         \~english Palette to use (0-15). Leave to 0 if in 256 color mode...
         \~french Palette  utiliser (0-15). Laisser  0 si en mode 256 couleurs.
    \param x
         \~english X position of the sprite
         \~french Position X du sprite
    \param y
         \~english Y position of the sprite
         \~french Position Y du sprite
*/
extern inline void PA_CreateSpriteFromGfx(bool screen, u8 obj_number, u16 obj_gfx, u8 obj_shape, u8 obj_size, u8 color_mode, u8 palette, s16 x, s16 y) {
   PA_obj[screen][obj_number].atr2 = obj_gfx + (palette << 12);
   PA_obj[screen][obj_number].atr0 = (y&OBJ_Y) + (color_mode << 13) + (obj_shape << 14);
   PA_obj[screen][obj_number].atr1 = (x & OBJ_X) + (obj_size << 14);
   ++obj_per_gfx[screen][obj_gfx];
};

/*! \fn extern inline void PA_CreateSpriteExFromGfx(bool screen, u8 obj_number, u16 obj_gfx, u8 obj_shape, u8 obj_size, u8 color_mode, u8 palette, u8 obj_mode, bool mosaic, bool hflip, bool vflip, u8 prio, bool dblsize, s16 x, s16 y)
    \brief
         \~english Create a sprite with it's gfx. This is the complex version of the function
         \~french Creer un sprite avec ses gfx... Ceci est la version complexe de la fonction
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj_number
         \~english Object number you want to use (0-127 for each screen seperately). 
         \~french Numro du sprite que vous voulez utiliser (de 0  127 pour chaque cran sparemment).
    \param obj_gfx
         \~english Memory gfx to use. Get it by using PA_GetSpriteGfx or PA_CreateGfx
         \~french Gfx en mmoire  utiliser. On peut en avoir avec PA_GetSpriteGfx ou PA_CreateGfx
    \param obj_shape
         \~english Object shape, from 0 to 2. Use the OBJ_SIZE_32X32 (...) macros for object shape and obj_size...
         \~french Forme du sprite  charger, de 0  2. Utiliser la macro OBJ_SIZE_32X32 (...) pour charger la forme et la taille...
    \param obj_size
         \~english Object size. Use the OBJ_SIZE_32X32 (...) macros for object shape and obj_size...
         \~french Taille du sprite. Utiliser la macro OBJ_SIZE_32X32 (...) pour charger la forme et la taille...
    \param color_mode
         \~english 256 or 16 color mode (1 or 0).
         \~french Mode 256 ou 16 couleurs (1 ou 0).
     \param palette
         \~english Palette to use (0-15). Leave to 0 if in 256 color mode...
         \~french Palette  utiliser (0-15). Laisser  0 si en mode 256 couleurs.
    \param obj_mode
         \~english Object mode (normal, transparent, window). Not functionnal yet, please leave to 0 for now
         \~french Mode du sprite (normal, transparent, fenetre). Pas encore oprationnel, laisser  0...
     \param mosaic
         \~english Activate Mosaic for the sprite or not. Not yet functionnal either :p
         \~french Activer le mode mosaique pour ce sprite. Pas encore au point...
    \param hflip
         \~english Horizontal flip on or off...
         \~french Flip horizontal activ ou non.
     \param vflip
         \~english Vertical flip...
         \~french Flip vertical...
    \param prio
         \~english Sprite priority regarding backgrounds : in front of which background to show it (0-3)
         \~french Priorit du sprite vis--vis des fonds : devant quel fond l'afficher... (0-3)
     \param dblsize
         \~english Double the possible sprite size. Activate only if you are going to rotate and zoom in the sprite
         \~french Doubler la taille possible du sprite. A activer uniquement si on compte grossir et faire tourner le sprite
    \param x
         \~english X position of the sprite
         \~french Position X du sprite
    \param y
         \~english Y position of the sprite
         \~french Position Y du sprite
*/
extern inline void PA_CreateSpriteExFromGfx(bool screen, u8 obj_number, u16 obj_gfx, u8 obj_shape, u8 obj_size, u8 color_mode, u8 palette, u8 obj_mode, bool mosaic, bool hflip, bool vflip, u8 prio, bool dblsize, s16 x, s16 y) {
   PA_obj[screen][obj_number].atr2 = obj_gfx + (prio << 10) + (palette << 12);
   PA_obj[screen][obj_number].atr0 = (y&OBJ_Y) + (dblsize << 9) + (obj_mode << 10) + (mosaic << 12) + (color_mode << 13) + (obj_shape << 14);
   PA_obj[screen][obj_number].atr1 = (x & OBJ_X) + (hflip << 12) + (vflip << 13) + (obj_size << 14);
   ++obj_per_gfx[screen][obj_gfx];
};


/*! \def PA_UpdateSpriteGfx(screen, obj_number, obj_data)
    \brief
         \~english Update the Gfx of a given sprite
         \~french Mettre  jour les Gfx d'un sprite donn
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj_number
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param obj_data
         \~english Gfx to load
         \~french Graphisme  charger
*/
#define PA_UpdateSpriteGfx(screen, obj_number, obj_data) PA_UpdateGfx(screen, PA_GetSpriteGfx(screen, obj_number), obj_data)

/*! \def PA_UpdateGfx(screen, gfx_number, obj_data)
    \brief
         \~english Update the Gfx of a given sprite
         \~french Mettre  jour les Gfx d'un sprite donn
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param gfx_number
         \~english Gfx number in memory
         \~french Numro du Gfx en mmoire
    \param obj_data
         \~english Gfx to load
         \~french Graphisme  charger
*/
#define PA_UpdateGfx(screen, gfx_number, obj_data) {DMA_Copy(obj_data, (void*)(SPRITE_GFX1 + (0x200000 *  screen) + (gfx_number << NUMBER_DECAL)), (used_mem[screen][gfx_number] << MEM_DECAL), DMA_32NOW);}



/*! \fn void PA_DeleteGfx(bool screen, u16 obj_gfx)
    \brief
         \~english Delete a given Gfx. If a sprite uses this gfx, it'll become invisible
         \~french Effacer un Gfx. Si un sprite l'utilisait, il deviendra invisible...
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj_gfx
         \~english Gfx number in memory
         \~french Numro du Gfx en mmoire
*/
void PA_DeleteGfx(bool screen, u16 obj_gfx);


/*! \fn void PA_DeleteSprite(bool screen, u8 obj_number)
    \brief
         \~english Delete a given sprite. If it is the only one to use it's gfx, it'll be deleted too.
         \~french Effacer un sprite. S'il tait le seul  utiliser un gfx, il sera effac lui aussi
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj_number
         \~english Sprite number
         \~french Numro du sprite
*/
void PA_DeleteSprite(bool screen, u8 obj_number);


/*! \def PA_SetSpriteRotEnable(screen, sprite, rotset)
    \brief
         \~english Rotate and zoom a sprite
         \~french Faire tourner et zoomer un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english Sprite you want to rotate
         \~french Sprite que l'on veut faire tourner
    \param rotset
         \~english Rotset you want to give to that sprite (0-31). You can apparently use a rotset for multiple sprites if zoomed/rotated identically...
         \~french Rotset que l'on veut pour un sprite donn (0-31). On peut a priori utiliser un rotset pour plusieurs sprites, s'ils sont zooms/tourns pareil...
*/

#define PA_SetSpriteRotEnable(screen, sprite, rotset) {PA_obj[screen][sprite].atr0 |= OBJ_ROT;  PA_obj[screen][sprite].atr1 = (PA_obj[screen][sprite].atr1 & ALL_BUT_ROTSET) + ((rotset) << 9);}

/*! \def PA_SetSpriteRotDisable(screen, sprite)
    \brief
         \~english Stop rotating and zooming a sprite
         \~french Arreter de faire tourner et zoomer un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english Sprite you want to rotate
         \~french Sprite que l'on veut faire tourner
*/
#define PA_SetSpriteRotDisable(screen, sprite) {PA_obj[screen][sprite].atr0 &= ALL_BUT(OBJ_ROT);  PA_obj[screen][sprite].atr1 &= ALL_BUT_ROTSET;}


/*! \fn extern inline void PA_SetRotset(bool screen, u8 rotset, s16 angle, u16 zoomx, u16 zoomy)
    \brief
         \~english Rotate and zoom a sprite
         \~french Faire tourner et zoomer un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param rotset
         \~english Rotset you want to change. To give a sprite a rotset, use PA_SetSpriteRotEnable...
         \~french Rotset que l'on veut changer. Pour attribuer un rotset  un sprite, utiliser PA_SetSpriteRotEnable
    \param angle
         \~english Angle, between 0 and 512 (not 360, be carefull)
         \~french Angle, entre 0 et 512 (et non 360, attention !)
    \param zoomx
         \~english Horizontal zoom. 256 is unzoomed, 512 is 2 times smaller, and 128 twice as big... So adjust at will ! :p
         \~french Zoom horizontal. 256 est pas de zoom, 512 2 fois plus petit, et 128 2 fois plus grand... Ajuster au mieux ! :p
    \param zoomy
         \~english Vertical zoom. 256 is unzoomed, 512 is 2 times smaller, and 128 twice as big... So adjust at will ! :p
         \~french Zoom vertical. 256 est pas de zoom, 512 2 fois plus petit, et 128 2 fois plus grand... Ajuster au mieux ! :p
*/

extern inline void PA_SetRotset(bool screen, u8 rotset, s16 angle, u16 zoomx, u16 zoomy) {
u8 obj_num = (rotset << 2);
	PA_obj[screen][obj_num].atr3 = (PA_Cos(angle) * zoomx) >> 8;
	PA_obj[screen][obj_num + 1].atr3 = (-PA_Sin(angle) * zoomy) >> 8;
	PA_obj[screen][obj_num + 2].atr3 = (PA_Sin(angle) * zoomx) >> 8;
	PA_obj[screen][obj_num + 3].atr3 = (PA_Cos(angle) * zoomy) >> 8;
}

/*! \fn extern inline void PA_SetRotsetNoZoom(bool screen, u8 rotset, s16 angle)
    \brief
         \~english Rotate a sprite without zooming. It's a bit faster than the normal PA_SetRotset function
         \~french Faire tourner un sprite sans zoomer. C'est un peu plus rapide que la fonction PA_SetRotset
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param rotset
         \~english Rotset you want to change. To give a sprite a rotset, use PA_SetSpriteRotEnable...
         \~french Rotset que l'on veut changer. Pour attribuer un rotset  un sprite, utiliser PA_SetSpriteRotEnable
    \param angle
         \~english Angle, between 0 and 512 (not 360, be carefull)
         \~french Angle, entre 0 et 512 (et non 360, attention !)
*/
extern inline void PA_SetRotsetNoZoom(bool screen, u8 rotset, s16 angle)   {
u8 obj_num = (rotset << 2);
	PA_obj[screen][obj_num].atr3 = PA_Cos(angle);
	PA_obj[screen][obj_num + 1].atr3 = -PA_Sin(angle);
	PA_obj[screen][obj_num + 2].atr3 = PA_Sin(angle);
	PA_obj[screen][obj_num + 3].atr3 = PA_Cos(angle);
}

/*! \fn extern inline void PA_SetRotsetNoAngle(bool screen, u8 rotset, u16 zoomx, u16 zoomy)
    \brief
         \~english Zoom a sprite without rotating. It's a bit faster than the normal PA_SetRotset function
         \~french Zoomer un sprite sans le faire tourner. C'est un peu plus rapide que la fonction PA_SetRotset
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param rotset
         \~english Rotset you want to change. To give a sprite a rotset, use PA_SetSpriteRotEnable...
         \~french Rotset que l'on veut changer. Pour attribuer un rotset  un sprite, utiliser PA_SetSpriteRotEnable
    \param zoomx
         \~english Horizontal zoom. 256 is unzoomed, 512 is 2 times smaller, and 128 twice as big... So adjust at will ! :p
         \~french Zoom horizontal. 256 est pas de zoom, 512 2 fois plus petit, et 128 2 fois plus grand... Ajuster au mieux ! :p
    \param zoomy
         \~english Vertical zoom. 256 is unzoomed, 512 is 2 times smaller, and 128 twice as big... So adjust at will ! :p
         \~french Zoom vertical. 256 est pas de zoom, 512 2 fois plus petit, et 128 2 fois plus grand... Ajuster au mieux ! :p
*/
extern inline void PA_SetRotsetNoAngle(bool screen, u8 rotset, u16 zoomx, u16 zoomy) {
u8 obj_num = (rotset << 2);
	PA_obj[screen][obj_num].atr3 = zoomx;
	PA_obj[screen][obj_num + 1].atr3 = 0;
	PA_obj[screen][obj_num + 2].atr3 = 0;
	PA_obj[screen][obj_num + 3].atr3 = zoomy;
}







/*! \def PA_SetSpriteX(screnn, obj, x)
    \brief
         \~english Set the X position of a sprite on screen
         \~french Position X du sprite  l'cran
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param x
         \~english X position
         \~french Position X
*/


#define PA_SetSpriteX(screen, obj, x) PA_obj[screen][obj].atr1 = (PA_obj[screen][obj].atr1 & ALL_BUT(OBJ_X)) + ((x) & OBJ_X)

/*! \def PA_GetSpriteX(screen, obj)
    \brief
         \~english Get the X position of a sprite on screen
         \~french Position X du sprite  l'cran
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
*/

#define PA_GetSpriteX(screen, obj) (PA_obj[screen][obj].atr1 & (OBJ_X))


/*! \def PA_SetSpriteY(screnn, obj, y)
    \brief
         \~english Set the Y position of a sprite on screen
         \~french Position Y du sprite  l'cran
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param y
         \~english Y position
         \~french Position Y
*/
#define PA_SetSpriteY(screen, obj, y) PA_obj[screen][obj].atr0 = (PA_obj[screen][obj].atr0 & ALL_BUT(OBJ_Y)) + ((y) & OBJ_Y)

/*! \def PA_GetSpriteY(screnn, obj)
    \brief
         \~english Get the Y position of a sprite on screen
         \~french Position Y du sprite  l'cran
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite

*/
#define PA_GetSpriteY(screen, obj) (PA_obj[screen][obj].atr0 & OBJ_Y)



/*! \fn extern inline void PA_SetSpriteXY(bool screen, u8 sprite, s16 x, s16 y)
    \brief
         \~english Set the X and Y position of a sprite on screen
         \~french Position X et Y du sprite  l'cran
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english sprite number in the sprite system
         \~french Numro du sprite dans le systeme de sprite
    \param x
         \~english X position
         \~french Position Y
    \param y
         \~english X position
         \~french Position Y
*/
extern inline void PA_SetSpriteXY(bool screen, u8 sprite, s16 x, s16 y) {
	PA_SetSpriteX(screen, sprite, x); 
	PA_SetSpriteY(screen, sprite, y);
}



/*! \def PA_SetSpritePal(screen, obj, pal)
    \brief
         \~english Set the 16 color palette used by a sprite
         \~french Changer la palette de 16 couleurs d'un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param pal
         \~english Palette number (0 - 15)
         \~french Numro de la palette (de 0  15)
*/
#define PA_SetSpritePal(screen, obj, pal) PA_obj[screen][obj].atr2 = (PA_obj[screen][obj].atr2 & ALL_BUT_PAL) + ((pal) << 12)

/*! \def PA_GetSpritePal(screen, obj)
    \brief
         \~english Get the 16 color palette used by a sprite
         \~french Palette de 16 couleurs d'un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
*/
#define PA_GetSpritePal(screen, obj) (PA_obj[screen][obj].atr2 >> 12)


/*! \def PA_SetSpriteDblsize(screen, obj, dblsize)
    \brief
         \~english Enable or disable double size for a given sprite
         \~french Activer ou dsactiver le mode Doublesize pour un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param dblsize
         \~english 1 to enable doublesize, 0 to disable it...
         \~french 1 pour l'activer, 0 pour l'inactiver
*/
#define PA_SetSpriteDblsize(screen, obj, dblsize) PA_obj[screen][obj].atr0 = (PA_obj[screen][obj].atr0 & ALL_BUT(DBLSIZE)) + ((dblsize) << 9)

/*! \def PA_GetSpriteDblsize(screen, obj)
    \brief
         \~english Get the double size state for a given sprite
         \~french Etat du mode Doublesize pour un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
*/
#define PA_GetSpriteDblsize(screen, obj) ((PA_obj[screen][obj].atr0 & DBLSIZE) >> 9)


/*! \def PA_SetSpriteColors(screen, sprite, n_colors)
    \brief
         \~english Change the sprite's color mode
         \~french Changer le mode de couleur du sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param n_colors
         \~english 0 for 16 colors, 1 for 256
         \~french 0 pour 16 couleurs, 1 pour 256
*/
#define PA_SetSpriteColors(screen, sprite, n_colors) PA_obj[screen][sprite].atr0 = (PA_obj[screen][sprite].atr0 & ALL_BUT(N_COLORS)) + ((n_colors) << 13)

/*! \def PA_GetSpriteColors(screen, sprite)
    \brief
         \~english Get a sprite's color mode
         \~french Mode de couleur d'un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
*/
#define PA_GetSpriteColors(screen, sprite) ((PA_obj[screen][sprite].atr0 & N_COLORS) >> 13)



/*! \def PA_SetSpriteMode(screen, sprite, obj_mode)
    \brief
         \~english Set the sprite's mode : 0 for normal, 1 for alpha blending, 2 for window
         \~french Rgler le mode d'un sprite : 0 pour normal, 1 pour transparent, 2 pour fenetre
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param obj_mode
         \~english Object mode : 0 for normal, 1 for alpha blending, 2 for window ; not working yet
         \~french Mode : 0 pour normal, 1 pour transparent, 2 pour fenetre ; ne marche pas encore
*/
#define PA_SetSpriteMode(screen, sprite, obj_mode) PA_obj[screen][sprite].atr0 = (PA_obj[screen][sprite].atr0 & ALL_BUT(OBJ_MODE)) + ((obj_mode) << 10)

/*! \def PA_GetSpriteMode(screen, obj)
    \brief
         \~english Get the sprite's mode : 0 for normal, 1 for alpha blending, 2 for window
         \~french Mode d'un sprite : 0 pour normal, 1 pour transparent, 2 pour fenetre
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
*/
#define PA_GetSpriteMode(screen, obj) ((PA_obj[screen][obj].atr0 & OBJ_MODE) >> 10)


/*! \def PA_SetSpriteMosaic(screen, obj, mosaic)
    \brief
         \~english Enable or disable mosaic mode for a given sprite
         \~french Mettre ou non un sprite en mode mosaic
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param mosaic
         \~english Set mosaic on (1) or off (0)
         \~french Mode mosaic activ (1) ou dsactiv (0)
*/
#define PA_SetSpriteMosaic(screen, obj, mosaic) PA_obj[screen][obj].atr0 = (PA_obj[screen][obj].atr0 & ALL_BUT(OBJ_MOSAIC)) + ((mosaic) << 12)

/*! \def PA_GetSpriteMosaic(screen, obj)
    \brief
         \~english Get the mosaic mode for a given sprite
         \~french Si un sprite est en mode mosaic
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
*/
#define PA_GetSpriteMosaic(screen, obj) ((PA_obj[screen][obj].atr0 & OBJ_MOSAIC) >> 12)



/*! \def PA_SetSpriteHflip(screen, obj, hflip)
    \brief
         \~english Enable or disable horizontal flip for a given sprite
         \~french Utiliser ou non le flip horizontal pour un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param hflip
         \~english Horizontal flip, 1 to enable, 0 to disable...
         \~french Flip horizontal, 1 pour oui, 0 pour non...
*/
#define PA_SetSpriteHflip(screen, obj, hflip) PA_obj[screen][obj].atr1 = (PA_obj[screen][obj].atr1 & ALL_BUT(OBJ_HFLIP)) + ((hflip) << 12)

/*! \def PA_GetSpriteHflip(screen, obj)
    \brief
         \~english Get the horizontal flip state for a given sprite
         \~french S'il y a un flip horizontal pour un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
*/
#define PA_GetSpriteHflip(screen, obj) ((PA_obj[screen][obj].atr1 & OBJ_HFLIP) >> 12)


/*! \def PA_SetSpriteVflip(screen, obj, vflip)
    \brief
         \~english Enable or disable vertical flip for a given sprite
         \~french Utiliser ou non le flip vertical pour un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param vflip
         \~english Vertical flip, 1 to enable, 0 to disable...
         \~french Flip vertical, 1 pour oui, 0 pour non...
*/
#define PA_SetSpriteVflip(screen, obj, vflip) PA_obj[screen][obj].atr1 = (PA_obj[screen][obj].atr1 & ALL_BUT(OBJ_VFLIP)) + ((vflip) << 13)

/*! \def PA_GetSpriteVflip(screen, obj)
    \brief
         \~english Get the  vertical flip state for a given sprite
         \~french Si le flip vertical est utilis ou non pour un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
*/
#define PA_GetSpriteVflip(screen, obj) ((PA_obj[screen][obj].atr1 & OBJ_VFLIP) >> 13)


/*! \def PA_SetSpriteGfx(screen, obj, gfx)
    \brief
         \~english Change the gfx used by a sprite
         \~french Modifier les graphismes utiliss par un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param gfx
         \~english Gfx number ; you can get one by using PA_CreateGfx or PA_GetSpriteGfx(obj_number);
         \~french Numro du gfx en mmoire ; on peut obtenir un numro avec PA_CreateGfx ou PA_GetSpriteGfx(obj_number);
*/
#define PA_SetSpriteGfx(screen, obj, gfx) PA_obj[screen][obj].atr2 = (PA_obj[screen][obj].atr2 & ALL_BUT(OBJ_GFX)) + ((gfx) & OBJ_GFX)

/*! \def PA_GetSpriteGfx(screen, obj)
    \brief
         \~english Get the gfx used by a sprite
         \~french Rcuprer le gfx utiliss par un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
*/
#define PA_GetSpriteGfx(screen, obj) (PA_obj[screen][obj].atr2 & OBJ_GFX)



/*! \def PA_SetSpritePrio(screen, obj, prio)
    \brief
         \~english Set a sprite's Background priority
         \~french Rgler la priorit d'un sprite par rapport au Bg.
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param prio
         \~english Sprite priority : 0 is over background 0, 1 over Bg 1, etc... (0-3)
         \~french Priorit du sprite : 0 est au-dessus du fond 0, 1 au-dessus du 1, etc... (0-3)
*/
#define PA_SetSpritePrio(screen, obj, prio) PA_obj[screen][obj].atr2 = (PA_obj[screen][obj].atr2 & ALL_BUT(OBJ_PRIO)) + ((prio) << 10)

/*! \def PA_GetSpritePrio(screen, obj)
    \brief
         \~english Get a sprite's Background priority
         \~french Rcuprer la priorit d'un sprite par rapport au Bg.
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
*/
#define PA_GetSpritePrio(screen, obj) ((PA_obj[screen][obj].atr2 & OBJ_PRIO) >> 10)


/*! \def PA_GetSpriteLx(screen, sprite)
    \brief
         \~english Get a sprite's length
         \~french Rcuprer la largeur d'un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
*/
#define PA_GetSpriteLx(screen, sprite) PA_size[PA_obj[screen][sprite].atr0 >> 14][PA_obj[screen][sprite].atr1 >> 14].lx


/*! \def PA_GetSpriteLy(screen, sprite)
    \brief
         \~english Get a sprite's height
         \~french Rcuprer la hauteur d'un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
*/
#define PA_GetSpriteLy(screen, sprite)PA_size[PA_obj[screen][sprite].atr0 >> 14][PA_obj[screen][sprite].atr1 >> 14].ly


/*! \def PA_CloneSprite(screen, obj, target)
    \brief
         \~english Clone a sprite. Works only for sprites on the same screen
         \~french Cloner un sprite. Marche uniquement pour les sprites sur un meme cran
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param obj
         \~english Object number in the sprite system
         \~french Numro de l'objet dans le systeme de sprite
    \param target
         \~english Target sprite to clone
         \~french Numro de la cible  cloner
*/
#define PA_CloneSprite(screen, obj, target) {PA_obj[screen][obj].atr0 = PA_obj[screen][target].atr0; PA_obj[screen][obj].atr1 = PA_obj[screen][target].atr1; PA_obj[screen][obj].atr2 = PA_obj[screen][target].atr2; ++obj_per_gfx[screen][PA_GetSpriteGfx(screen, target)];}






/*! \fn extern inline void PA_InitSpriteAnim(bool screen, u8 sprite, void* graphics)
    \brief
         \~english Initialise the animation system for a given sprite. Use only if your sprite was not created with PA_CreateSprite or PA_CreateSpriteEx
         \~french Initialise le systeme d'animation pour un sprite donn. A utiliser uniquement si le sprite n'a pas t cr avec PA_CreateSprite ou PA_CreateSpriteEx
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english sprite number in the sprite system
         \~french Numro du sprite dans le systeme de sprite
    \param graphics
         \~english Graphics used by the sprite
         \~french Graphismes utilis par le sprite
*/
extern inline void PA_InitSpriteAnim(bool screen, u8 sprite, void* graphics){
	PA_SpriteAnimP[screen][sprite] = (u16*)graphics;
}




/*! \fn extern inline void PA_SetSpriteAnimEx(bool screen, u8 sprite, u8 lx, u8 ly, u8 ncolors, s16 animframe)
    \brief
         \~english Set the animation frame for a given sprite. This function is faster than the normal PA_SetSpriteAnim because it doesn't have to lookup the sprite dimensions...
         \~french Rgler l'image du sprite dans l'animation. Cette fonction est plus rapide que PA_SetSpriteAnim parce qu'elle n'a pas  rechercher les dimensions du sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english sprite number in the sprite system
         \~french Numro du sprite dans le systeme de sprite
    \param lx
         \~english Sprite width (8, 16, 32, 64)
         \~french Largeur du sprite (8, 16, 32, 64)
    \param ly
         \~english Sprite height (8, 16, 32, 64)
         \~french Hauteur du sprite (8, 16, 32, 64)		 
    \param ncolors
         \~english Sprite color mode (0 for 16 colors, 1 for 256)
         \~french Mode couleur du sprite (0 pour 16 couleurs, 1 pour 256)
    \param animframe
         \~english Sprite animation frame (0, 1, 2, etc...)
         \~french Frame de l'animation du sprite (0, 1, 2, etc...)
*/
extern inline void PA_SetSpriteAnimEx(bool screen, u8 sprite, u8 lx, u8 ly, u8 ncolors, s16 animframe){
	PA_UpdateSpriteGfx(screen, sprite, (void*)(PA_SpriteAnimP[screen][sprite] + (animframe * (lx * ly) >> (2 - ncolors))));
}

/*! \fn extern inline void PA_SetSpriteAnim(bool screen, u8 sprite, s16 animframe)
    \brief
         \~english Set the animation frame for a given sprite. Same as PA_SetSpriteAnimEx, but a bit slower and easier to use...
         \~french Rgler l'image du sprite dans l'animation. Identique  PA_SetSpriteAnimEx, mais plus simple  utiliser, par contre plus lent
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english sprite number in the sprite system
         \~french Numro du sprite dans le systeme de sprite	 
    \param animframe
         \~english Sprite animation frame (0, 1, 2, etc...)
         \~french Frame de l'animation du sprite (0, 1, 2, etc...)
*/
extern inline void PA_SetSpriteAnim(bool screen, u8 sprite, s16 animframe){
	PA_SetSpriteAnimEx(screen, sprite, PA_GetSpriteLx(screen, sprite), PA_GetSpriteLy(screen, sprite), PA_GetSpriteColors(screen, sprite), animframe);
}








/*! \fn void PA_SetSpritePixelEx(bool screen, u8 sprite, u8 hsize, u8 n_colors, u8 x, u8 y, u8 color)
    \brief
         \~english Set a sprite's pixel to a given palette color
         \~french Mettre un pixel d'un sprite  une couleur donne
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english Sprite number in the sprite system
         \~french Numro du sprite dans le systeme de sprite
    \param hsize
         \~english Horizontal size '8, 16, 32...)
         \~french Taille horizontale (8, 16...)
    \param n_colors
         \~english 0 or 1 for 16 or 256 colors...
         \~french 0 ou 1 pour 16 ou 256 couleurs
    \param x
         \~english X coordinate of the pixel to change
         \~french Coordonne X du pixel  changer
    \param y
         \~english Y coordinate of the pixel to change
         \~french Coordonne Y du pixel  changer
    \param color
         \~english New palette color to put
         \~french Nouvelle couleur de la palette  metrre
*/
void PA_SetSpritePixelEx(bool screen, u8 sprite, u8 hsize, u8 n_colors, u8 x, u8 y, u8 color);



/*! \def PA_SetSpritePixel(screen, sprite, x, y, color)
    \brief
         \~english Set a sprite's pixel to a given palette color. Like PA_SetSpritePixelEx, with less options, but a little slower
         \~french Mettre un pixel d'un sprite  une couleur donne. Comme PA_SetSpritePixelEx, avec moins d'options, mais un peu plus lent
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english Sprite number in the sprite system
         \~french Numro du sprite dans le systeme de sprite
    \param x
         \~english X coordinate of the pixel to change
         \~french Coordonne X du pixel  changer
    \param y
         \~english Y coordinate of the pixel to change
         \~french Coordonne Y du pixel  changer
    \param color
         \~english New palette color to put
         \~french Nouvelle couleur de la palette  metrre
*/
#define PA_SetSpritePixel(screen, sprite, x, y, color) PA_SetSpritePixelEx(screen, sprite, PA_GetSpriteLx(screen, sprite), PA_GetSpriteColors(screen, sprite), x, y, color)


/*! \fn u8 PA_GetSpritePixelEx(bool screen, u8 sprite, u8 hsize, u8 n_colors, u8 x, u8 y)
    \brief
         \~english Get a sprite's pixel color
         \~french Rcuprer la couleur d'un pixel d'un sprite
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english Sprite number in the sprite system
         \~french Numro du sprite dans le systeme de sprite
    \param hsize
         \~english Horizontal size '8, 16, 32...)
         \~french Taille horizontale (8, 16...)
    \param n_colors
         \~english 0 or 1 for 16 or 256 colors...
         \~french 0 ou 1 pour 16 ou 256 couleurs
    \param x
         \~english X coordinate of the pixel
         \~french Coordonne X du pixel
    \param y
         \~english Y coordinate of the pixel
         \~french Coordonne Y du pixel
*/
u8 PA_GetSpritePixelEx(bool screen, u8 sprite, u8 hsize, u8 n_colors, u8 x, u8 y);


/*! \def PA_GetSpritePixel(screen, sprite, x, y)
    \brief
         \~english Get a sprite's pixel color. Like PA_GetSpritePixelEx, with less options, but a little slower
         \~french Rcuprer la couleur d'un pixel d'un sprite. Comme PA_GetSpritePixelEx, avec moins d'options, mais un peu plus lent
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english Sprite number in the sprite system
         \~french Numro du sprite dans le systeme de sprite
    \param x
         \~english X coordinate of the pixel
         \~french Coordonne X du pixel
    \param y
         \~english Y coordinate of the pixel
         \~french Coordonne Y du pixel
*/
#define PA_GetSpritePixel(screen, sprite, x, y) PA_GetSpritePixelEx(screen, sprite, PA_GetSpriteLx(screen, sprite), PA_GetSpriteColors(screen, sprite), x, y)


/*! \fn void PA_InitSpriteDraw(bool screen, u8 sprite, u8 draw_number, u8 drawsize)
    \brief
         \~english Initialise a sprite to be able to draw on it !
         \~french Initialise un sprite pour pouvoir dessiner dessus ! 
    \param screen
         \~english Chose de screen (0 or 1)
         \~french Choix de l'cran (0 ou 1)
    \param sprite
         \~english Sprite number in the sprite system
         \~french Numro du sprite dans le systeme de sprite
    \param draw_number
         \~english Sprite number you want to use in the drawing system (0-15)
         \~french Numro du sprite que l'on veut dans le systeme de dessin (0-15)
    \param drawsize
         \~english Size of the pixel when drawing... 0 for 1 pixel, 1 for 2 pixels, etc... 
         \~french Taille du trait quand on dessine. 0 pour 1 pixel, 1 pour 2...
*/
void PA_InitSpriteDraw(bool screen, u8 sprite, u8 draw_number, u8 drawsize);


/*! \fn void PA_SpriteDraw(u8 draw_number, s16 x, s16 y, u16 color)
    \brief
         \~english Draw on a given Drawable sprite... Must be used every frame, or else use PA_SpriteDrawNot if you are not drawing...
         \~french Dessiner sur un sprite intialis... Doit etre utilis  tous les tours, sinon il faut faire PA_SpriteDrawNot si on ne dessine pas...
    \param draw_number
         \~english Number of the drawable sprite
         \~french Numro du sprite dessinable
    \param x
         \~english X position on the sprite
         \~french Position X sur le sprite
    \param y
         \~english Y position on the sprite
         \~french Position Y sur le sprite
    \param color
         \~english Palette drawing color
         \~french Couleur du trait de la palette
*/
void PA_SpriteDraw(u8 draw_number, s16 x, s16 y, u16 color);


/*! \def PA_SpriteDrawNot(draw_number)
    \brief
         \~english Must be used if a drawing sprite is initialised and you are not drawing on it
         \~french Doit etre mis si un sprite dessinable est initialis et que l'on ne dessine pas dessus actuellement
    \param draw_number
         \~english Number of the drawable sprite
         \~french Numro du sprite dessinable
*/
#define PA_SpriteDrawNot(draw_number)  PA_DrawSprite[draw_number].wasdrawing = 0












/** @} */ // end of Sprite


























#endif


